@@ -1,6 +1,6 @@
 package Sub::Spec::Gen::ReadTable;
 BEGIN {
-  $Sub::Spec::Gen::ReadTable::VERSION = '0.01';
+  $Sub::Spec::Gen::ReadTable::VERSION = '0.02';
 }
 # ABSTRACT: Generate function (and its spec) to read table data
 
@@ -229,7 +229,7 @@ _
 }
 
 sub _parse_query {
-    my ($args, $table_spec, $col_specs, $col2arg) = @_;
+    my ($args, $opts, $table_spec, $col_specs, $col2arg) = @_;
     my $query = {args=>$args};
 
     my @columns = keys %$col_specs;
@@ -319,7 +319,35 @@ sub _parse_query {
     }
     $query->{filters}       = \@filters;
     $query->{filter_fields} = \@filter_fields;
-    $query->{q}             = $args->{q};
+
+    my @searchable_fields = grep {
+        !defined($col_specs->{$_}{attr_hashes}[0]{column_searchable}) ||
+            $col_specs->{$_}{attr_hashes}[0]{column_searchable}
+        } @columns;
+    my $search_opts = {ci => $opts->{case_insensitive_search}};
+    my $search_re;
+    my $q = $args->{q};
+    if (defined $q) {
+        if ($opts->{word_search}) {
+            $search_re = $opts->{case_insensitive_search} ?
+                qr/\b$q\b/i : qr/\b$q\b/;
+        } else {
+            $search_re = $opts->{case_insensitive_search} ?
+                qr/$q/i : qr/$q/;
+        }
+    }
+    $query->{q} = $args->{q};
+    $query->{search_opts} = $args->{search_opts};
+    unless ($opts->{custom_search}) {
+        $query->{search_fields} = \@searchable_fields;
+        $query->{search_str_fields} = [grep {
+            $col_specs->{$_}{type} =~ /^(str)$/
+        } @searchable_fields];
+        $query->{search_array_fields} = [grep {
+            $col_specs->{$_}{type} =~ /^(array)$/
+        } @searchable_fields];
+        $query->{search_re} = $search_re;
+    }
 
     my @sort_fields;
     my @sorts;
@@ -374,7 +402,8 @@ sub _gen_func {
                 unless ref($args{fields}) eq 'ARRAY';
         }
 
-        my $res = _parse_query(\%args, $table_spec, $col_specs, $col2arg);
+        my $res = _parse_query(
+            \%args, $opts, $table_spec, $col_specs, $col2arg);
         return $res unless $res->[0] == 200;
         my ($query) = @{$res->[2]};
 
@@ -404,22 +433,8 @@ sub _gen_func {
         no warnings; # silence undef warnings when comparing row values
 
         $log->tracef("(read_table_func) Filtering ...");
-
-        my @search_fields = grep {
-            $col_specs->{$_}{type} =~ /^(str)$/
-        } @columns;
-        my $search_opts = {ci => $opts->{case_insensitive_search}};
-        my $search_re;
         my $q = $query->{q};
-        if (defined $q) {
-            if ($opts->{word_search}) {
-                $search_re = $opts->{case_insensitive_search} ?
-                    qr/\b$q\b/i : qr/\b$q\b/;
-            } else {
-                $search_re = $opts->{case_insensitive_search} ?
-                    qr/$q/i : qr/$q/;
-            }
-        }
+        my $search_re = $query->{search_re};
 
       ROW:
         for my $row0 (@$data) {
@@ -477,14 +492,22 @@ sub _gen_func {
             if (defined $q) {
                 if ($opts->{custom_search}) {
                     next ROW unless $opts->{custom_search}->(
-                        $row_h, $q, $search_opts);
+                        $row_h, $q, $query->{search_opts});
                 } else {
                     my $match;
-                    for my $f (@search_fields) {
+                    for my $f (@{$query->{search_str_fields}}) {
                         if ($row_h->{$f} =~ $search_re) {
                             $match++; last;
                         }
                     }
+                  ARY_FIELD:
+                    for my $f (@{$query->{search_array_fields}}) {
+                        for my $el (@{$row_h->{$f}}) {
+                            if ($el =~ $search_re) {
+                                $match++; last ARY_FIELD;
+                            }
+                        }
+                    }
                     next ROW unless $match;
                 }
             }
@@ -602,9 +625,10 @@ separated by comma.
 
 ** *q* => STR
 
-A filtering option. By default, all fields will be searched using simple
-case-insensitive string search. In the future, a method to customize searching
-will be allowed.
+A filtering option. By default, all fields except those specified with
+column_searchable=0 will be searched using simple case-insensitive string
+search. There are a few options to customize this, using these gen arguments:
+*word_search*, *case_insensitive_search*, and *custom_search*.
 
 ** Filter arguments
 
@@ -788,7 +812,7 @@ Sub::Spec::Gen::ReadTable - Generate function (and its spec) to read table data
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 SYNOPSIS
 
@@ -945,9 +969,10 @@ separated by comma.
 
 ** *q* => STR
 
-A filtering option. By default, all fields will be searched using simple
-case-insensitive string search. In the future, a method to customize searching
-will be allowed.
+A filtering option. By default, all fields except those specified with
+column_searchable=0 will be searched using simple case-insensitive string
+search. There are a few options to customize this, using these gen arguments:
+*word_search*, *case_insensitive_search*, and *custom_search*.
 
 ** Filter arguments
 
